# coding=utf-8

import argparse
import codecs
import sys

import netmagus.session

if __name__ == '__main__':
    # BASEDIR = os.path.dirname(inspect.getabsfile(inspect.currentframe()))
    if sys.stdout.encoding != 'UTF-8':
        sys.stdout = codecs.getwriter('utf-8')(sys.stdout, 'strict')
    # handle command line arguments
    parser = argparse.ArgumentParser(
        description='NetMagus NetworkOperation Script')
    parser.add_argument('--script',
                        help='name of the user Python file to execute',
                        required=True)
    parser.add_argument('--input-file',
                        help='full path to a valid NetMagus NetworkOperation '
                             'form JSON output file', required=True)
    parser.add_argument('--token',
                        help='hash key sent by NetMagus to indicate the '
                             'instance of this formula '
                             'execution.  Used by formula to generate RPC '
                             'targets and/or file names for '
                             'responses', required=True)
    parser.add_argument('--loglevel',
                        help='integer 0-5 indicating desired logging level',
                        required=True)
    cli_args = parser.parse_args()
    netmagus.session.NetMagusSession(token=cli_args.token,
                                     input_file=cli_args.input_file,
                                     loglevel=cli_args.loglevel,
                                     script=cli_args.script).start()



# coding=utf-8
from __future__ import absolute_import, division, print_function, \
    unicode_literals

import contextlib
import importlib
import json
import logging
import os
import shelve
import sys
import traceback

import netmagus.form
import netmagus.rpc


class NetMagusSession(object):
    """
    This class is a wrapper to handle an interactive session between the
    NetMagus backend and a user script.  It serves as a unifed API for user
    script operations to send/receive data with the NetMagus backend.  It
    also serves as a unified API for the NetMagus backend to call and execute
    user scripts.

    The NetMagus backend will call the module as (ex.):
        "python -m netmagus --script user.py --token aBcDeF --input-file
        /some/path/to/aBcDeF.json --loglevel 10

    The "token" is used by NM to desginate all file and RPC interactions tied
    to a given execution.

    This class will be called by the module's __main__.py file to:
     - read JSON from the NetMagus backend via the input-file if it exists
     - read in any previous state tied to the token/session
     - import the user's script/module and execute it's run() method
        - run() method must receive a NetMagusSession object as only arg
        - run() method may return Form or (Form, anyobject)
     - receive a Form (and a state object) as a return from the user's
       run() method
     - store any state object and send JSON response back to NetMagus backend


     The .start method is used to initiate the execution of the user's code
     in the formula.

     The user's code can use various attributes and methods of this session
     object to interact with the NetMagus server and its UI.

     The following attributes exist within each session object:

     nm_input:  holds the data entered by end-user on the NM ui
     user_state: holds any state data from previous formula steps

     The following methods exist within each session object for interaction
     with the NetMagus server:

     rpc_connect: opens a real-time RPC bus to the NM server
     rpc_disconnect: closes the formula's RPC connection to the NM server
     rpc_send:  sends a messsage to the NM server via RPC using this formula's
                current execution token
     rpc_receive:   retrieves any data pending on NM server for this formula's
                    execution token
     rpc_form_query:    sends a form to UI via RPC call and blocks until user
                        submits a response or timeout is exceeded

     The following objects can be sent to the NetMagus server for display in
     the UI:

     rpc_html:  used to send HTML based notices real-time to NM UI for display
                to the end-user
     form:  used to send a form composed of form component objects to ask an
            end-user for input

     The following objects within the sesion can be sent to the NetMagus UI
     inside a form object:

     textarea
     textinput
     radiobutton
     passwordinput
     dropdownmenu
     checkbox

    """

    _log_level_index = {
        0: 0,
        1: logging.DEBUG,
        2: logging.INFO,
        3: logging.WARN,
        4: logging.ERROR,
        5: logging.CRITICAL
    }

    def __init__(self, token, input_file, loglevel, script):
        """
        the launcher will simply parse CLI args and instantiate a new
        NetMagusSession. This will need to receive those CLI args and store
        to internal object state
        """
        self.token = token
        self.input_file = input_file
        self.loglevel = self._log_level_index[int(loglevel)]
        self.script = script
        self.logger = None
        self.nm_input = None
        self.user_state = None

        # Convenience methods for user script writers
        self.rpc_connect = netmagus.rpc.rpc_connect
        self.rpc_disconnect = netmagus.rpc.rpc_disconnect
        self.rpc_form = netmagus.rpc.Form
        self.rpc_html = netmagus.rpc.Html
        self.netopstep = netmagus.form.Form  # for legacy <v0.6.0 compatability
        self.form = netmagus.form.Form
        self.textarea = netmagus.form.TextArea
        self.textinput = netmagus.form.TextInput
        self.radiobutton = netmagus.form.RadioButton
        self.passwordinput = netmagus.form.PasswordInput
        self.dropdownmenu = netmagus.form.DropDownMenu
        self.checkbox = netmagus.form.CheckBox

    def rpc_send(self, message):
        """
        Convenience method to handle the token manipulation for the session.
        See rpc.rpc_send method for full args
        """
        netmagus.rpc.rpc_send(self.token, message)

    def rpc_receive(self):
        """
        Convenience method to handle the token manipulation for the session.
        See rpc.rpc_receive method for full args
        """
        return netmagus.rpc.rpc_receive(self.token)

    def rpc_form_query(self, message, **kwargs):
        """
        Convenience method to handle token manipulation for the session
        :param message: an rpc.Message object to be sent to NetMagus backend
        :param kwargs: see rpc.rpc_form_query for full arg list
        :return:
        """
        return netmagus.rpc.rpc_form_query(self.token, message, **kwargs)

    def start(self):
        """
        Method to start the execution of user code in the user Formula
        :return:
        """
        # if no logging was requested, disabled all output in the logging module
        if not self.loglevel:
            logging.disable(logging.CRITICAL)
        else:
            # Set the logging level here for all steps
            logging.basicConfig(level=self.loglevel)
            self.logger = logging.getLogger(__name__)
            self.logger.setLevel(self.loglevel)

        self.__read_response_file()
        self.__read_state_file()
        formula_return = self.__run_user_script()
        self.__write_response_file(formula_return)
        self.__write_state_file(self.user_state)

    def __run_user_script(self):
        try:
            if not os.path.isfile(self.script):
                raise IOError('Path to user script must be a full absolute '
                              'path, not just a file name.')
            sys.path.append(os.path.dirname(self.script))
            module_name = os.path.basename(self.script).split('.')[0]
            self.logger.debug('Importing user module {}'.format(module_name))
            user_module = importlib.import_module(module_name)
            self.logger.debug(
                    '{} imported user module named: {}'.format(
                            type(user_module),
                            user_module))
            formula_return = user_module.run(self)
            if isinstance(formula_return, netmagus.form.Form):
                next_step = formula_return
                self.__fix_formcomponent_indexes(next_step)
                # user must pass full file path back to us for the next
                # script he  wants executed, we add the rest
                next_step.commandPath = ' '.join(
                        [sys.executable, '-m netmagus --script',
                         next_step.commandPath])
            elif isinstance(formula_return, tuple):
                next_step, formula_state = formula_return
                self.__fix_formcomponent_indexes(next_step)
                next_step.commandPath = ''.join(
                        [self.script, next_step.commandPath])
            else:
                raise TypeError(
                        'Formula files should return either Form or tuple('
                        'Form, stateobject)')
            return formula_return
        except ImportError:
            self.logger.critical(
                    'Unable to load the specified python module: {}'.format(
                            self.script))
            raise
        except (Exception, NameError, IOError) as ex:
            self.logger.critical(
                    'Error calling run() method defined in the target file: '
                    '{}'.format(self.script))
            tb = traceback.format_exc()
            logging.exception(ex)
            htmlextrainfo = {
                'html': {
                    'printToUser': True,
                    'outputType': 'html',
                    'title': 'ERROR IN FORMULA',
                    'data': '<h3>This formula has encountered a critical error '
                            'and can not continue.  Please review the '
                            'traceback info and/or contact support for '
                            'assistance.</h3><br><br>Traceback info was: '
                            '<pre>{}</pre>'.format(tb)
                }
            }
            next_step = netmagus.form.Form(currentStep=1,
                                           dataValid=False,
                                           extraInfo=htmlextrainfo)

        return next_step

    def __read_response_file(self):
        """
        Read in the JSON response file from the NetMagus back-end.  These files
        are generated each time the NetMagus backend executes a commandPath to
        launch a task defined in a formula.  Examples would be when a user
        presses the SUBMIT button in the UI, a JSON file is generated by
        NetMagus and stored in a temp file to pass data to the Formula.
        """
        self.logger.debug('Reading JSON data from NetMagus request')
        try:
            with open(self.input_file) as data_file:
                self.nm_input = json.load(data_file)
            os.remove(self.input_file)  # remove file after reading it
        except IOError:
            self.logger.warn('Unable to access input JSON file {}'.format(
                    self.input_file))
        except TypeError:
            self.logger.error('Unable to decode JSON data in {}'.format(
                    self.input_file))

    def __read_state_file(self):
        """
        This method will retrieve any previous state data saved during this
        formula execution and store it internal as self.user_state.
        """
        state_file = self.input_file + '_State'
        with contextlib.closing(
                shelve.open(state_file, protocol=2, writeback=True)) \
                as state_shelve:
            try:
                self.user_state = state_shelve['formula_state']
                self.logger.debug('Formula state retrieved from shelf in {}'.format(state_file))
            except KeyError:
                self.logger.info(
                        'The file {} exists but does not contain previous '
                        'state from a formula execution.  '
                        'Setting state to NONE'.format(state_file))
                self.user_state = None
            except IOError:
                self.logger.info(
                        'No _State file found from previous formula execution '
                        'steps. Setting state to NONE.')
                self.user_state = None
            except TypeError:  # handle case where user module is in py2.7 without unicode literals
                try:
                    self.user_state = state_shelve[b'formula_state']
                    self.logger.debug('Formula state retrieved from shelf in {}'.format(state_file))
                except KeyError:
                    self.logger.info(
                        'The file {} exists but does not contain previous '
                        'state from a formula execution.  '
                        'Setting state to NONE'.format(state_file))
                    self.user_state = None
                except IOError:
                    self.logger.info(
                        'No _State file found from previous formula execution '
                        'steps. Setting state to NONE.')
                    self.user_state = None

    def __write_response_file(self, response):
        # store the return data into a Response file for NetMagus to
        # continue the op if using file-based exchange
        response_file = self.input_file + 'Response'
        self.logger.debug(
                'Target output JSON file will be: {}'.format(response_file))
        try:
            with open(response_file, 'w') as outfile:
                outfile.write(str(response))
        except IOError:
            self.logger.error(
                    'Unable to open target JSON Response file: {}'.format(
                            response_file))
            raise

    def __write_state_file(self, stateobject):
        """
        store the returned state object into a file for future operation
        steps to retrieve.  Formula creators can return any object in a tuple
        along with their normal Form object fo the next screen in the
        format of (Form, anyobject) to have anyobject saved to disk and passed
        to the next formula execution step where it will be retrieved and stored
        in NetMagusSession.user_state for use by other formula code
        """
        state_file = self.input_file + '_State'
        self.logger.debug(
                'Target output shelve file will be: {}'.format(state_file))
        try:
            with contextlib.closing(shelve.open(state_file, protocol=2,
                                                writeback=True)) as \
                    state_shelve:
                state_shelve['formula_state'] = stateobject
                self.logger.debug('Formula state shelved in {}'.format(state_file))
        except IOError:
            self.logger.error(
                    'Unable to open target shelve state file: {}'.format(
                            state_file))
            raise
        except TypeError:  # handle case where user module is in py2.7 without unicode literals
            try:
                with contextlib.closing(shelve.open(state_file, protocol=2,
                                                    writeback=True)) as \
                        state_shelve:
                    state_shelve[b'formula_state'] = stateobject
                    self.logger.debug('Formula state shelved in {}'.format(state_file))
            except IOError:
                self.logger.error(
                    'Unable to open target shelve state file: {}'.format(
                        state_file))
                raise

    @staticmethod
    def __fix_formcomponent_indexes(form_obj):
        """
        This method is a temporary fix to append an index attribute to each
        form component to be sent to NetMagus as JSON.  Eventually this will
        be done in the NetMagus back-end upon receipt according to the order
        of the list of form controls.  For now these are being added here in
        the same fashion before being sent to the NetMagus back-end.
        :param form_obj: a netmagus.NetOpStep object to be serialized and
        sent to NetMagus
        """
        # TODO: REMOVE INDEX INCREMENTING ONCE PROBERT100 FIXES NM
        index_counter = 0
        for item in form_obj.form:
            setattr(item, 'index', index_counter)
            index_counter += 1



# coding=utf-8
from __future__ import absolute_import, division, print_function, \
    unicode_literals

import time

import autobahn_sync

from netmagus.form import Form

"""
NetMagus Python Library
Copyright (C) 2016 Intelligent Visbility, Inc. - Richard Collins
<richardc@intelligentvisibility.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

"""
Normally a Form (as defined in the netmagus.form module) is used to
represent individual NetMagus UI screens
and does data exchange with formula code via file exchange.

Within a given formula, a user may choose to have multiple real-time data
exchanges between the NetMagus UI and the formula itself.  To do this,
the below RPC methods and data object classes can be used to do WAMP based
RPC calls to pass messages to/from the UI instead of using file-based JSON
exchanges.

A sample usage pattern to send a new screen would be:
- usernameinput = netmagus.form.TextInput(label='username')
- message_to_user = netmagus.rpc.Form(name="Username Input",
description="Enter your user name below",
form=[usernameinput])
- netmagus.rpc.rpc_connect('ws://nmhostaddress:8080/ws', 'netmagus')
- username = netmagus.rpc.rpc_form_query(token, myformobject, poll=.5,
timeout=60)
    - here the UI will update and show the form, and wait for user to enter
    the data and press submit
    - once user presses submit the response will be waitig for rcp_receive to
    pick it up
    - rpc_form_query polls the RPC looking for a reponse from the user at
    'poll' interval and stop at 'timeout' if no repsonse received
- use normal try logic to determine if timeout occurred by catching the
RpcCallTimeout exception and handling it

A user can write these as a series of synchronous interactions with the UI to
allow a single formula step to gather numerous pieces of information,
make branched execution decisions, or collapse many formula steps into one.

This allows formula writers to avoid having to adopt async programming
constructs to interact with the NetMagus UI since the typical NetMagus user
experience is a step-by-step "wizard" like approach with code blocks executed
between each interaction.

To display simple HTML updates to a user, use pattern such as:
mymessage = netmagus.rpc.Html(append=True, title='Important User Update!',
data="<p>Processing your input now...</p>")
netmagus.rpc.rpc_send(token, mymessage)

This will fire a RPC message to the UI which will then display the data
within the user's screen
"""


class Message(object):
    """
    RPC Message Envelope

    Example:
        {
         'messageType': 'html',
         'message': {}
         }

    messageType: property inferred by data type sent in the message arg
        possible values:
            html: used for html content
            form: used for asking user a question
    message: object containing actual message of type netmagus.rpc.Form or
    netmagus.rpc.Html
    """

    def __init__(self, message):
        self.message = message

    def __repr__(self):
        return str(self.as_dict)

    @property
    def as_dict(self):
        """
        return a dictionary representation of the Message() with proper values
        for message type and proper dictionary version of the message payload
        """
        return {
            'messageType': self.messageType, 'message': self.message.as_dict
        }

    @property
    def messageType(self):
        """
        Returns proper message type based upon the data type inside the message
        """
        if isinstance(self.message, Form):
            return 'netop'
        elif isinstance(self.message, Html):
            return 'html'


class Html(object):
    """
    HTML object used for sending rich content back to the user
     {
          'printToUser': True,
          'append': True,
          'outputType': 'html',
          'title': 'Just an RPC Call',
          'data': "<b>Executing command please wait</b> <i title='Executing
          Operation' class='glyph-icon  fa
          fa-spinner icon-spin'></i>"
     }
    printToUser: boolean flag controls if this is sdisplayed (this may not be
    needed anymore)
    append: controls if the data should be appended to exsiting html output
    or replace what is already there
    outputType: type of data to be displayed (this may not be needed anymore)
    title: displayed in the header section
    data: actual html code
    """

    def __init__(self, printToUser=True, append=True, title="", data=""):
        self.printToUser = printToUser
        self.append = append
        self.title = title
        self.data = data
        self.outputType = 'html'

    def __repr__(self):
        return str(self.as_dict)

    @property
    def as_dict(self):
        """
        Return a dictionary representation of self
        :return: dictionary of self
        """
        return self.__dict__


def rpc_connect(url="ws://127.0.0.1:8088/ws", realm="netmagus"):
    """
    Utilizes autobahn_sync to create Twisted thread in the background to
    handle async WAMP messaging and connect to a given WAMP URL and realm.
    WAMP is used by NetMagus for real-time data exchange between the UI and
    any wizards/formulas to update a UI screen or retrive data entered by a
    user.

    :param url: WAMP URL to connet to (ex - Crossbar.io instance @
    ws://localhost:8088/ws)
    :param realm: the crossbar realm to use for this connection (ex - netmagus)
    """
    return autobahn_sync.run(url=url, realm=realm)


def rpc_disconnect():
    """
    Stop the autobahn_sync Twisted thread
    """
    return autobahn_sync.app.stop()


def rpc_send(token, message, retry_count=10, retry_delay=.2):
    """
    This method is used to send data to NetMagus for a given wizard/formula
    execution.  Each execution has its own RPC methods created upon execution
    to allow RTC between the UI and the formula/wizard code.  After using
    rpc_send, rpc_receive can be used to fetch results once the operation is
    complete.
    This method by default will retry sending the data for 2 seconds it in
    cases were target method is not registered yet or in cases where the
    back-end does not acknowledge our receipt.  After time-out, it then will
    raise an RpcCallTimeout exception.

    :param token: random/unique token supplied by NetMagus for each execution
    of a formula/wizard.  It is used to associate messages with individual
    formula execution instances.
    :param message: the payload to be sent to
    :param retry_delay:  How long in seconds to wait between retry_count
    :param retry_count: How many times to retry the send operation until a valid
                    ACK is received or until no exception is raised
    :return: None - to receive results/response, use rpc_receive with same token
    """
    if isinstance(message, Form):
        rpc_target = "com.intelligentvisibility." + token + ".browser.sendform"
    elif isinstance(message, Html):
        rpc_target = "com.intelligentvisibility." + token + \
                     ".browser.displaymessage"
    else:
        raise TypeError(
                'invalid rpc_send Message payload, must be of type '
                'netmagus.rpc.Form or netmagus.rpc.Html')
    while retry_count >= 0:
        try:
            response = autobahn_sync.call(rpc_target, Message(message).as_dict)
            if response == "ok":
                return response
            retry_count -= 1
            time.sleep(retry_delay)
        except autobahn_sync.ApplicationError as exc:
            if "com.intelligentvisibility" + token + ".ui." in \
                    exc.error_message():
                # TODO: remove once NM stops passing these from browser ui
                return "ok"
            if retry_count <= 0:
                raise
            retry_count -= 1
            time.sleep(retry_delay)
    raise RpcCallTimeout


def rpc_receive(token, retry_count=10, retry_delay=.2):
    """
    Since NetMagus RPC calls may have an indeterminite amount of time before
    a user completes an interaction and submits the response data,
    rpc_receive is used as a polling mechanism to allow formula writers to
    avoid having to write async code to handle responses. This method will
    :param token: random/unique token supplied by NetMagus for each execution
    of a formula/wizard.  It is used to associate messages with individual
    formula execution instances.
    :param retry_count: the # of times we should try to retrieve data via the
    WAMP RPC call at the RPC target
    :param retry_delay: the amount of time in seconds between retries
    :return: None if no data ready, else whatever JSON response data is
    available from the the UI
    """
    while retry_count >= 0:
        try:
            return autobahn_sync.call(
                    "com.intelligentvisibility." + token +
                    ".browser.getformresponse")
        except autobahn_sync.ApplicationError:
            if retry_count <= 0:
                raise
            retry_count -= 1
            time.sleep(retry_delay)
    raise RpcCallTimeout


class RpcCallTimeout(Exception):
    """Custom Exception for handling timeouts when handling timeouts while
    waiting for poll-based synchronous WAMP
    RPC calls to return"""
    pass


def rpc_form_query(token, message, poll=1, timeout=-1):
    """
    Send a Form to the Netmagus UI, then poll the response target looking for
    the return value from a user. This method blocks indefinitely unless a
    timeout value is provided.  Poll frequency dictates how often additional
    WAMP RPC calls are made looking for the response data from the NetMagus UI.
    :param token: session token passed down from NetMagus UI when it runs a
    formula
    :param message:
    :param poll: poll interval in seconds to look for response data
    :param timeout: timeout in seconds to block waiting for a response,
    0 disables block, -1 blocks indefinitely
    :return:
    """
    if isinstance(message, Form):
        rpc_send(token, message)
        start_time = time.time()
        response = rpc_receive(token)
        while not response and timeout > 0:
            time.sleep(poll)
            response = rpc_receive(token)
            if time.time() > (start_time + timeout):
                raise RpcCallTimeout(
                        'Exceeded timeout of {}s waiting for response to call '
                        'for '
                        'token {}.  '
                        'Message was: {}'.format(timeout, token, message))
        while not response and timeout < 0:
            time.sleep(poll)
            response = rpc_receive(token)
        return response
    else:
        raise TypeError(
                'invalid Message payload, must be of type netmagus.rpc.Form')



# coding=utf-8
"""
NetMagus Python Library
Copyright (C) 2016 Intelligent Visbility, Inc.
Richard Collins <richardc@intelligentvisibility.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

import copy
import json


class Form(object):
    """
    Form objects are sent to NetMagus back-end to be rendered in the UI.
    This controls the FormComponents seen on the screen, the behavior of
    buttons, and activities performed by the backend.
    """

    def __init__(self, name='', commandPath='', description='', form=None,
                 extraInfo=None, currentStep=1,
                 dataValid=True, buttonName='', saveRecord=True, dynamic=False,
                 autoProceed=False,
                 disableBackButton=False, finalStep=False):
        """
        A NetMagus forumla typically behaves as a "wizard" style user
        interaction.  Each "step" or "screen" in in the wizard is created
        using a JSON representation of the screen atributes and controls.
        This class allows a user to wrap numerous netmagus.form controls and
        control the user interaction, shell command exeuctions, and more.

        A JSON serialized repr of this object will control the individual
        screens and interactions of the NetMagus
        HTML UI.

        :param name: The name field is displayed at the top of the NetMagus
        UI screen as a typical HTML header
        :param commandPath: this is the full path of the shell command to be
        executed when a user presses NEXT or SUBMIT
        :param description: This may be text or html and will be rendered
        appropriately in the UI.  It is displayed
        below the name field.
        :param form: a list containing one or more netmagus.form objects to
        be drawn below the description in the UI
        :param extraInfo: optional dictionary to be rendered as additional
        JSON data to be used by UI to display extra HTML messages to the
        user.  It may also be used to pass data to UI to be received in
        future commandPath executuions.
        :param currentStep: integer representing the # of this operation in
        the overall wizard (i.e. 1 for first screen, 2 for 2nd, etc.)
        :param dataValid: boolean to indicate if the data received from
        NetMagus from the user's input was valid or not
        :param buttonName: string to control the name of the button displayed
        to users when using RPC based messages
        :param saveRecord: boolean to indicate if this screen/step should be
        saved in the UI history
        :param dynamic: boolean to specify if this screen contains controls
        with dynamic data that must be refreshed when a user presses the BACK
        button.  If True, commandPath is executed again when a user revisits
        the screen
        :param autoProceed: boolean to indicate if the UI should autoproceed
        from its current screen to this new one sent to the UI.  if False,
        user must manually press the PROCEED button to advance to the next
        screen/step
        :param disableBackButton: boolean to grey out the back button in the
        NetMagus UI for a given Form inside a forumula
        :param finalStep: boolean to indicate when a screen is the last/final
        screen in an operation.  This is used by NetMagus UI to understand
        when a task is finished and that it no longer needs to have its
        execution tracked by the execution engine
        """
        self.id = 0
        self.name = name
        self.description = description
        self.currentStep = currentStep
        self.commandPath = commandPath
        self.form = form or []
        self.extraInfo = extraInfo or {}
        self.dataValid = dataValid
        self.buttonName = buttonName
        self.saveRecord = saveRecord
        self.dynamic = dynamic
        self.autoProceed = autoProceed
        self.disableBackButton = disableBackButton
        self.finalStep = finalStep

    def __repr__(self):
        return json.dumps(self, sort_keys=True, default=lambda o: o.__dict__)

    @property
    def as_dict(self):
        """
        return a dictionary representation of the Form object an all of its
        FormComponent objects inside the self.form attribute
        :return: dict(self) where self.form=list(dict(FormComponent))
        """
        return_dictionary = copy.copy(self.__dict__)
        return_dictionary['form'] = [component.as_dict for component in
                                     self.form]
        return return_dictionary


class FormComponent(object):
    """
    Parent class for all valid types of form elements.  This should be
    subclassed for each new type of component created in NetMagus forms.
    This class is not intended for direct instantiation.

    The NetMagus app may choose to use or ignore values depending upon type
    of control.  The base class is set to provide all current attributes with
    a default value.  Sub-classes should over-ride defaults as necessary for
    each control type implemented.

    :param component:  Valid form component string value
    :param editable: Boolean to control if form component may be edited by user
    :param label: String rendered to left of the form component
    :param description: String rendered below the form component
    :param placeholder: String rendered inside the component before user
    inputs their own data
    :param options: List containing choices for checkboxes, dropdowns, etc.
    :param required: Boolean that controls if user must provide a value for
    this form component
    :param validation: String passed to NetMagus for certain component types
    to enforce user input validation validation examples: number=[number],
    email=[email], url=[url],  none=/.*/
    """

    def __init__(self, component='', editable=True, label='', description='',
                 placeholder='', options=None,
                 required=False, validation='/.*/', value=None):
        # common attrs for all components
        self.component = component
        self.editable = editable
        self.index = 0
        self.label = label
        self.description = description
        self.required = required
        if options is None:
            self.options = []
        else:
            self.options = options
        self.placeholder = placeholder
        self.validation = validation
        self.value = value

    def __repr__(self):
        return json.dumps(self, sort_keys=True, default=lambda o: o.__dict__)

    @property
    def as_dict(self):
        return self.__dict__


class TextArea(FormComponent):
    def __init__(self, editable=True, label='', description='',
                 placeholder='', required=False):
        super(TextArea, self).__init__(component='textArea', editable=editable,
                                       label=label,
                                       description=description,
                                       required=required,
                                       placeholder=placeholder)


class TextInput(FormComponent):
    def __init__(self, editable=True, label='', description='', placeholder='',
                 required=False, validation='/.*/'):
        super(TextInput, self).__init__(component='textInput',
                                        editable=editable, label=label,
                                        description=description,
                                        required=required,
                                        placeholder=placeholder,
                                        validation=validation)


class CheckBox(FormComponent):
    def __init__(self, editable=True, label='', description='', options=None,
                 required=False, value=None):
        super(CheckBox, self).__init__(component='checkbox', editable=editable,
                                       label=label,
                                       description=description, options=options,
                                       required=required,
                                       value=value)


class RadioButton(FormComponent):
    def __init__(self, editable=True, label='', description='', options=None,
                 required=False, value=None):
        super(RadioButton, self).__init__(component='radio', editable=editable,
                                          label=label,
                                          description=description,
                                          options=options, value=value,
                                          required=required)


class DropDownMenu(FormComponent):
    def __init__(self, editable=True, label='', description='', options=None,
                 value=None, required=False):
        super(DropDownMenu, self).__init__(component='select',
                                           editable=editable, label=label,
                                           description=description,
                                           options=options, value=value,
                                           required=required)


class PasswordInput(FormComponent):
    def __init__(self, editable=True, label='', description='', placeholder='',
                 required=False):
        super(PasswordInput, self).__init__(component='password',
                                            editable=editable, label=label,
                                            description=description,
                                            required=required,
                                            placeholder=placeholder)



# coding=utf-8
"""
NetMagus Python Library
Copyright (C) 2016 Intelligent Visbility, Inc.
Richard Collins <richardc@intelligentvisibility.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import absolute_import, division, \
    print_function, unicode_literals

import netmagus.form

# create reference for old name for backwards compatability with <v0.6.0
NetOpStep = netmagus.form.Form



# coding=utf-8
from setuptools import find_packages, setup

setup(name='netmagus',
      version='0.6.2',
      description='Python module for JSON data exchange via files or RPC with the '
                  'Intelligent Visibility NetMagus system.',
      url='http://www.intelligentvisibility.com/netmagus/',
      author='Richard Collins',
      author_email='richardc@intelligentvisibility.com',
      license='MIT',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Intended Audience :: Information Technology',
          'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
          'Natural Language :: English',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Topic :: System :: Networking'
      ],
      keywords='netmagus network automation netops',
      packages=find_packages(exclude=['tests*', 'examples']),
      install_requires=['future','autobahn-sync']
      )
